@@ -1,6 +1,6 @@
 package DBI::Gofer::Execute;
 
-#   $Id: Execute.pm 9668 2007-06-21 21:40:01Z timbo $
+#   $Id: Execute.pm 10087 2007-10-16 12:42:37Z timbo $
 #
 #   Copyright (c) 2007, Tim Bunce, Ireland
 #
@@ -16,10 +16,10 @@ use DBI::Gofer::Response;
 
 use base qw(DBI::Util::_accessor);
 
-our $VERSION = sprintf("0.%06d", q$Revision: 9668 $ =~ /(\d+)/o);
+our $VERSION = sprintf("0.%06d", q$Revision: 10087 $ =~ /(\d+)/o);
 
 our @all_dbh_methods = sort map { keys %$_ } $DBI::DBI_methods{db}, $DBI::DBI_methods{common};
-our %all_dbh_methods = map { $_ => DBD::_::db->can($_) } @all_dbh_methods;
+our %all_dbh_methods = map { $_ => (DBD::_::db->can($_)||undef) } @all_dbh_methods;
 
 our $local_log = $ENV{DBI_GOFER_LOCAL_LOG}; # do extra logging to stderr
 
@@ -41,6 +41,7 @@ my %configuration_attributes = (
     forced_connect_attributes  => {},
     track_recent => 1,
     check_request_sub => sub {},
+    check_response_sub => sub {},
     forced_single_resultset => 1,
     max_cached_dbh_per_drh => 1,
     max_cached_sth_per_dbh => 1,
@@ -286,6 +287,13 @@ sub execute_request {
             : $self->execute_dbh_request($request);
     };
     $response ||= $self->new_response_with_err(undef, $@, $current_dbh);
+
+    if (my $check_response_sub = $self->check_response_sub) {
+        # not protected with an eval so it can choose to throw an exception
+        my $new = $check_response_sub->($response, $self, $request);
+        $response = $new if ref $new;
+    }
+
     undef $current_dbh;
 
     $response->warnings(\@warnings) if @warnings;
@@ -301,7 +309,7 @@ sub execute_dbh_request {
     my $dbh;
     my $rv_ref = eval {
         $dbh = $self->_connect($request);
-        my $args = $request->dbh_method_call; # [ 'method_name', @args ]
+        my $args = $request->dbh_method_call; # [ wantarray, 'method_name', @args ]
         my $wantarray = shift @$args;
         my $meth      = shift @$args;
         $stats->{method_calls_dbh}->{$meth}++;
@@ -610,7 +618,7 @@ sub _mk_rand_callback {
         #no warnings 'uninitialized';
         #warn "_mk_rand_callback($fail_percent:$fail_modrate, $delay_percent:$delay_modrate): seqn=$seqn fail=$fail delay=$delay";
         if ($delay) {
-            my $msg = "DBI_GOFER_RANDOM delaying execution of $method by $delay_duration seconds\n";
+            my $msg = "DBI_GOFER_RANDOM delaying execution of $method() by $delay_duration seconds\n";
             # Note what's happening in a trace message. If the delay percent is an odd
             # number then use warn() so it's sent back to the client
             ($delay_percent % 2 == 0) ? $h->trace_msg($msg) : warn($msg);
@@ -618,7 +626,7 @@ sub _mk_rand_callback {
         }
         if ($fail) {
             undef $_; # tell DBI to not call the method
-            return $h->set_err(1, "fake error induced by DBI_GOFER_RANDOM env var");
+            return $h->set_err($DBI::stderr, "fake error from $method method induced by DBI_GOFER_RANDOM env var ($fail_percent%)");
         }
         return;
     }
@@ -626,7 +634,7 @@ sub _mk_rand_callback {
 
 
 sub update_stats {
-    my ($self, $request, $response, $frozen_request, $frozen_response, $time_received) = @_;
+    my ($self, $request, $response, $frozen_request, $frozen_response, $time_received, $meta) = @_;
 
     my $stats = $self->{stats};
     $stats->{frozen_request_max_bytes} = length($frozen_request)
@@ -641,6 +649,7 @@ sub update_stats {
             response => $frozen_response,
             time_received => $time_received,
             duration => dbi_time()-$time_received,
+	    ($meta) ? (meta => $meta) : (), # for any other info
         };
         shift @$recent_requests if @$recent_requests > $track_recent;
     }
@@ -678,7 +687,7 @@ Examples include: L<DBI::Gofer::Transport::stream> and L<DBI::Gofer::Transport::
 =head2 check_request_sub
 
 If defined, it must be a reference to a subroutine that will 'check' the request.
-It is pass the request object and the executor as its only arguments.
+It is passed the request object and the executor as its only arguments.
 
 The subroutine can either return the original request object or die with a
 suitable error message (which will be turned into a Gofer response).
@@ -686,6 +695,15 @@ suitable error message (which will be turned into a Gofer response).
 It can also construct and return a new request that should be executed instead
 of the original request.
 
+=head2 check_response_sub
+
+If defined, it must be a reference to a subroutine that will 'check' the response.
+It is passed the response object, the executor, and the request object.
+The sub may alter the response object and return undef, or return a new response object.
+
+This mechanism can be used to, for example, terminate the service if specific
+database errors are seen.
+
 =head2 forced_connect_dsn
 
 If set, this DSN is always used instead of the one in the request.
@@ -811,7 +829,7 @@ Each method has a distinct sequence number.
 
 =head1 AUTHOR
 
-Tim Bunce, L<http://www.linkedin.com/in/timbunce>
+Tim Bunce, L<http://www.tim.bunce.name>
 
 =head1 LICENCE AND COPYRIGHT
 
